// Author: Matthew Shiel
package main

import (
	"embed"
	"fmt"
	"io/fs"
	"net/http"

	"git.riomhaire.com/gremlin/elizaservice/eliza"

	"github.com/jaffee/commandeer"
)

//go:embed web/*
var templates embed.FS

type Main struct {
	BotName string `help:"What This Bot is Called."`
	Port    int    `help:"What port bot is listening too"`
}

func NewMain() *Main {
	return &Main{
		BotName: "Eliza",
		Port:    8070,
	}
}

func inputhandler(w http.ResponseWriter, r *http.Request) {
	// Extract question from GET request
	question := r.URL.Query().Get("value")

	// Return Eliza's response's so long as the user doesn't give a quit statement
	answer := eliza.ReplyTo(question)

	// Return Eliza's answer
	fmt.Fprintf(w, "%s", answer)
}

func (m *Main) Run() error {

	fmt.Println(`
####### #         ###   #######    #
#       #          #         #    # #
#       #          #        #    #   #
#####   #          #       #    #     #
#       #          #      #     #######
#       #          #     #      #     #
####### #######   ###   ####### #     #
	`)
	fmt.Println()
	// create file server handler - but need to strip off prefix
	fsys, err := fs.Sub(templates, "web")
	if err != nil {
		panic(err)
	}
	webroot := http.FileServer(http.FS(fsys))

	http.Handle("/", webroot)

	// Handles the user input and return of Eliza's answers
	http.HandleFunc("/user-input", inputhandler)
	fmt.Printf(" [*] %s ... Awaiting Conversations on port  %d\n", m.BotName, m.Port)
	http.ListenAndServe(fmt.Sprintf(":%d", m.Port), nil)

	return nil
}

func main() {
	err := commandeer.Run(NewMain())
	if err != nil {
		fmt.Println(err)
	}
}
